
// Kick things off.
$(document).ready(function(){
	$('#picker button').click(function(){
		var c = document.getElementById('can');
		var ctx = c.getContext('2d');
		var i = document.getElementById('i');
		ctx.clearRect(0,0,c.width,c.height);
		ctx.drawImage(renderTile(
			$('#a').is(':checked'),
			$('#b').is(':checked'),
			$('#c').is(':checked'),
			$('#d').is(':checked'),
			i
		),0,0);
		return false;
	
	});
});

/**
 * Manipulates a base texture and draws a tile in the desired
 * configuration.
 * @param {boolean} a Is the top-left corner raised?
 * @param {boolean} b Is the top-right corner raised?
 * @param {boolean} c Is the bottom-right corner raised?
 * @param {boolean} d Is the bottom-left corner raised?
 * @param {tile} The texture to use. Returned tile will be double the size.
 */
function renderTile(a,b,c,d,tile){
	// Make up a new canvas to play on.
	var can = document.createElement('canvas');
	can.width = tile.width*2;
	can.height = tile.height*2;
	var ctx = can.getContext('2d');
	ctx.save();
	
	// Also create a buffer so we can manipulate textures before drawing
	var canBuff = document.createElement('canvas');
	canBuff.width = 200;
	canBuff.height = 200;
	var ctxBuff = canBuff.getContext('2d');
	
	// Just to make things more readable.
	var combo = String(a)+b+c+d;
	
	// Here'es the transform(s) for each elevation combo.
	if(combo == 'truetruefalsefalse'){
		// 1100
		ctx.transform(1,-.5,1,1,0,tile.height);
		ctx.drawImage(tile,0,0);
	}else if(combo == 'falsetruetruefalse'){
		// 0110
		ctx.transform(1,-1,1,.5,0,tile.height*1.5);
		ctx.drawImage(tile,0,0);
	}else if(combo == 'falsefalsetruetrue'){
		// 0011
		ctx.transform(1,-.5,1,0,0,tile.height*2);
		ctx.drawImage(tile,0,0);
	}else if(combo == 'truefalsefalsetrue'){
		// 1001
		ctx.transform(1,0,1,.5,0,tile.height*1.5);
		ctx.drawImage(tile,0,0);
	}else if(combo == 'falsetruefalsefalse'){
		// 0100
		ctx.transform(1,-.5,1,.5,0,tile.height*1.5);
		ctx.drawImage(tile,0,0);
		ctx.restore();
		
		ctxBuff.transform(1,-1,1,1,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		
		ctx.drawImage(canBuff,0,0,tile.width*2,tile.height*1.5,0,0,tile.width*2,tile.height*1.5);
	} else if(combo=='falsefalsetruefalse'){
		// 0010
		ctx.transform(1,-1,1,0,0,tile.height*2);
		ctx.drawImage(tile,0,0);
		ctx.restore();
		ctx.clearRect(0,tile.height,tile.width,tile.height);
		
		ctxBuff.transform(1,-.5,1,.5,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		ctx.drawImage(canBuff,0,tile.height,tile.width,tile.height,0,tile.height,tile.width,tile.height);
	} else if(combo=='truefalsefalsefalse'){
		// 1000
		ctx.transform(1,0,1,1,0,tile.height);
		ctx.drawImage(tile,0,0);
		ctx.restore();
		ctx.clearRect(tile.width,tile.height,tile.width,tile.height);
		
		ctxBuff.transform(1,-.5,1,.5,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		ctx.drawImage(canBuff,tile.width,tile.height,tile.width,tile.height,tile.width,tile.height,tile.width,tile.height);
	} else if(combo=='truetruetruefalse'){
		// 1110
		ctx.transform(1,-.5,1,1,0,tile.height);
		ctx.drawImage(tile,0,0);
		ctx.restore();
		ctx.clearRect(tile.width,0,tile.width,tile.height*2);
		
		ctxBuff.transform(1,-1,1,.5,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		ctx.drawImage(canBuff,tile.width,0,tile.width,tile.height*2,tile.width,0,tile.width,tile.height*2);
	} else if(combo=='truetruefalsetrue'){
		// 1101
		ctxBuff.save();
		ctxBuff.transform(1,-.5,1,1,0,tile.height);
		ctxBuff.drawImage(tile,0,0);
		
		/* Mask the tile so as to only draw the required portion */
		ctxBuff.globalCompositeOperation = 'destination-in';
		ctxBuff.beginPath();
		ctxBuff.moveTo(0,0);
		ctxBuff.lineTo(tile.width,0);
		ctxBuff.lineTo(tile.width,tile.height);
		ctxBuff.fill();
		
		ctxBuff.restore();
		ctx.drawImage(canBuff,0,0);
		
		ctxBuff.transform(1,0,1,.5,0,tile.height);
		ctxBuff.drawImage(tile,0,0);
		ctxBuff.globalCompositeOperation = 'destination-in';
		
		ctxBuff.beginPath();
		ctxBuff.moveTo(0,0);
		ctxBuff.lineTo(tile.width,tile.height);
		ctxBuff.lineTo(0,tile.height);
		ctxBuff.fill();
		
		ctx.drawImage(canBuff,0,0);
		
	} else if(combo == 'truefalsetruetrue'){
		// 1011
		ctxBuff.save();
		ctxBuff.transform(1,0,-1,.5,tile.width,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		
		/* Mask the tile so as to only draw the required portion */
		ctxBuff.globalCompositeOperation = 'destination-in';
		ctxBuff.beginPath();
		ctxBuff.moveTo(tile.width,0);
		ctxBuff.lineTo(0,0);
		ctxBuff.lineTo(tile.width,tile.height);
		ctxBuff.fill();
		
		ctxBuff.restore();
		ctx.drawImage(canBuff,0,0);
		
		
		ctxBuff.transform(1,0,1,.5,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		
		/* Mask the tile so as to only draw the required portion */
		ctxBuff.globalCompositeOperation = 'destination-in';
		ctxBuff.beginPath();
		ctxBuff.moveTo(0,tile.height);
		ctxBuff.lineTo(0,0);
		ctxBuff.lineTo(tile.width,0);
		ctxBuff.fill();
		
		ctx.drawImage(canBuff,0,0);
	} else if(combo == 'falsetruetruetrue'){
		// 0111
		ctxBuff.save();
		ctxBuff.transform(1,-1,1,.5,0,tile.height*1.5);
		ctxBuff.drawImage(tile,0,0);
		
		/* Mask the tile so as to only draw the required portion */
		ctxBuff.globalCompositeOperation = 'destination-in';
		ctxBuff.beginPath();
		ctxBuff.moveTo(tile.width,0);
		ctxBuff.lineTo(0,0);
		ctxBuff.lineTo(tile.width,tile.height);
		ctxBuff.fill();
		
		ctxBuff.restore();
		ctx.drawImage(canBuff,0,0);
		
		
		ctxBuff.save();
		ctxBuff.transform(1,0,-1,.5,tile.width,tile.height);
		ctxBuff.drawImage(tile,0,0);
		
		/* Mask the tile so as to only draw the required portion */
		ctxBuff.globalCompositeOperation = 'destination-in';
		ctxBuff.beginPath();
		ctxBuff.moveTo(tile.width,0);
		ctxBuff.lineTo(tile.width,tile.height);
		ctxBuff.lineTo(0,tile.height);
		ctxBuff.fill();
		
		ctx.drawImage(canBuff,0,0);
	} else {
		// combo == 'falsefalsefalsefalse'
		// 0000
		// 0001
		ctx.transform(1,-.5,1,.5,0,tile.height*1.5);
		ctx.drawImage(tile,0,0);
	}
	
	return can;
	
}
