/**
 * Game Menu
 * Make a nested list style menu based on a game object.
 * @param $target The target HTML element to append to
 * @param options The options object. Manatory values include ""
 */
var GameMenu = function($target, options){
	
	this.target = $target;
	$.extend(this,options);
	var menu = this;
	
	function menuClick(){
		var ref = $(this).data('ref');
		$(this)
			.closest('.level-0')
			.find('ul')
			.not($(this).next('ul'))
			.not($(this).closest('ul'))
			.trigger('slideLeft');
		
		$(this)
			.parent()
			.siblings()
				.removeClass('selected')
				.end();
		
		if(!_.isUndefined(ref.sub)){
			$(this)
				.next()
				.trigger('slideRight');
		}
		if(!_.isUndefined(ref.action)){
			menu.currentElement = $(this);
			if(ref.action.call(menu,ref) !== false){
				$(this).parent().addClass('selected');
			}
		}
		return false;
	}

	function hideMenu(){
		$(this)
			.parent()
			.removeClass('selected');
		return $(this)
			.not('.level-0')
			.css('width',0)
			.css('padding-left',0)
			.css('padding-right',0)
			.css('border-width',0);
	}
	function showMenu(){
		$(this)
			.parent()
			.addClass('selected');
		return $(this)
			.css('width',menu.iconSize+'px')
			.css('padding-left',menu.paddingSize+'px')
			.css('padding-right',menu.paddingSize+'px')
			.css('border-width','1px');
	}

	function makeList(buttons,level){
		var level = _.isUndefined(level) ? 0 : level;
		var $interface = $('<ul></ul>').addClass('menu level-'+level);
		
		$.each(buttons,function(index,button){
			button.name = index;
			var $link = $('<a href="#">')
				.data('ref',button)
				.click(menuClick)
				.attr('title',index);
				
			var $newButton = $('<li></li>');
			
			if(!_.isUndefined(button.icon)){
				$link
					.addClass('icon')
					.append(
					$('<img>')
						.attr('src',button.icon)
						.attr('alt',index)
				);
			} else {
				$link
					.addClass('text')
					.text(index);
			}
			$newButton.append($link);
			if(!_.isUndefined(button.sub)){
				$newButton.append(makeList(button.sub,level+1).trigger('slideLeft'));
			}
			$interface.append($newButton);
		});
		
		$interface.bind('slideLeft',this,hideMenu);
		$interface.bind('slideRight',this,showMenu);
		return $interface;
	}

	$(this.target).append(makeList(this.menuData));
}

GameMenu.prototype = {
	setTool : function(toolClass,toolSub){
		this.toolClass = toolClass;
		this.toolSub = toolSub;
	},
	setIcon : function(src){
		this.currentElement.find('img').attr('src',src);
	}
}
