var IsoCan = function(canvas,options){
	
	// inits this.canvas
	if(!this.getRenderContext(canvas)) {
		return false;
	}
	
	// Apply any custom parameters.
	_.extend(this,options);
	
	this.map = [];
	this.defaultTile = IsoRes.Tiles.Tex;
	this.setTileDef(256,256);
	this.setMapDef(4,4);
	this.setOffset(0,0);
	this.setZoom(1);
	
	this.gameSpeed = 0;
	this.gameTime = 0;
	this.maxStep = 0.05;
	this.wallLastTimestamp = 0;
	
	var _it = this;
	
	//create event handlers.
	var handlers = {};
	_.each(['start','stop','move','click'],function(action){
		handlers[action] = function(e){return _it.handlerClick(e,action);}
	});
	
	// and set them
	_.each({
		onmousedown : handlers.start,
		onmouseup : handlers.stop,
		onmouseout : handlers.stop,
		onmousemove : handlers.move,
		onclick : handlers.click,
		ontouchstart : handlers.start,
		ontouchstop : handlers.stop,
		ontouchmove : handlers.move
	},function(fn,event){
		_it.canvasElement[event] = fn;
	});
	
	// Setup mouse wheel zooming.
	var wheel = function(event){
		if(
			event.wheelDelta && event.wheelDelta > 0
			||
			event.detail && event.detail < 0
		){
			_it.callHook('mapZoom',{e:event,dir:true});
			_it.zoomIn();
		} else {
			_it.callHook('mapZoom',{e:event,dir:false});
			_it.zoomOut();
		}
	}
	
	if (window.addEventListener)
		window.addEventListener('DOMMouseScroll', wheel, false);
	window.onmousewheel = wheel;
	
	this.canvasElement.style.cursor="move";
	
}

IsoCan.prototype = {
	
	version : 0,
	cursorPos:[0,0],
	baseUrl : '',
	map : [],
	offset : [0,0],
	zoom : {
		max : .25,
		min : 8,
		current : 1
	},
	
	Filters : {
		savegameSave : [],
		savegameLoad : [],
	},
	
	Hooks : {
		redrawMapBeforeTiles : [],
		redrawMapBeforeSprites : [],
		redrawMapBeforeDraw : [],
		redrawMapAfter : [],
		redrawMapDrawTile : [],
		rendererSpriteBeforeDraw : [],
		mapClick : [],
		mapMove : [],
		mapZoom : [],
	},
	
	/**
	 * Apply specified filters.
	 * Loop over the filters object specified and if there's any
	 * filters, apply them to this content.
	 * @param {string} filter Name of the filter.
	 * @param {mixed} A piece of content which the filter(s) are to process.
	 * @return {mixed} The filtered output.
	 */
	applyFilter : function(filter,content){
		if(_.isObject(this.Filters[filter])){
			for(var i=0; i < this.Filters[filter].length; i++){
				content = this.Filters[filter][i].call(this,content);
			}
		}
		return content;
	},
	
	/**
	 * Call selected hook.
	 * Loop over the filters object specified and if there's any
	 * hooks set, call them.
	 * @param {string} filter Name of the hook.
	 * @return undefined
	 */
	callHook : function(hook,args){
		if(_.isObject(this.Hooks[hook])){
			for(var i=0; i < this.Hooks[hook].length; i++){
					this.Hooks[hook][i].call(this, args);
			}
		}
	},
	
	getImageUri : function(tileName,tileClass,uriClass,format){
		var format = _.isUndefined(format) ? 'png' : format;
		var uriClass = _.isUndefined(uriClass) ? 'tiles' : uriClass;
		return this.baseUrl+uriClass+'/'+tileName+'/'+tileClass+'.'+format;
	},
	
	setTileDef : function(w,h,subtiles){
		var subtiles = _.isUndefined(subtiles) ? 8 : subtiles;
		this.tileDef = {
			width : w,
			height: h,
			tilesPerX : w/2,
			tilesPerY : h/4,
			subtiles : subtiles
		};
		return this;
	},
	
	setMapDef : function(w,h,replace){
		this.mapDef = {
			widthTiles : w,
			heightTiles : h,
			heightPx : h * this.tileDef.tilesPerY,
			widthPx : w * this.tileDef.tilesPerX,
		}
		
		if(replace != false || _.isUndefined(replace)){
			for(var x=0; x<w; x++){
				this.map[x] = [];
				for(var y=0; y<h; y++){
					this.setTile(x,y,this.createTile('default'));
				}
			}
		}
		return this;
	},
	
	createTile : function(tileName,options){
		options = _.extend({},options,{parent:this});
		var newTile = tileName == 'default' ? new this.defaultTile(options) : new IsoRes.Tiles[tileName](options);
		newTile.prepareTile();
		return newTile;
	},
	createSprite : function(spriteName,options){
		options = _.extend({},options,{parent:this});
		var newSprite = new IsoRes.Sprites[spriteName](options);
		this.sprites.push(newSprite);
		return newSprite;
	},	
	setTile : function(x,y,tile,stack){
		tile.x = x;
		tile.y = y;
		if(stack === false || !_.isObject(this.map[x][y])){
			this.map[x][y] = [];
		}
		
		for(var z=0; z<this.map[x][y].length; z++){
			if(!this.map[x][y][z].canStack(tile)) {
				return false;
			}
		}
		
		this.map[x][y].push(tile);
		return tile;
	},
	
	clearTile : function(x,y){
		this.setTile(x,y,this.createTile('default'),false)
	},
	
	setOffset : function(x,y){
		this.hasOffsetChanged = true;
		this.offset = [x,y];
	},
	
	adjustOffset : function(xDiff,yDiff){
		// Now set the values.
		this.hasOffsetChanged = true;
		this.offset[0]+= xDiff*this.zoom.current;
		this.offset[1]+= yDiff*this.zoom.current;
		return this;
	},
	
	zoomIn : function(){
		var level= this.getZoom()/2;
		if(this.isZoomWithinBounds(level)){
			this
				.setZoom(level)
				.adjustOffset(this.canvasElement.width/2,this.canvasElement.height/2);
			this.refreshBuffer();
		}
		return this;
	},
	
	zoomOut : function(){
		var level= this.getZoom()*2;
		if(this.isZoomWithinBounds(level)){
			this
				.adjustOffset(-this.canvasElement.width/2,-this.canvasElement.height/2)
				.setZoom(this.getZoom()*2);
			this.refreshBuffer();
		}
		return this;
	},
	
	isZoomWithinBounds : function(level){
		if(level >= this.zoom.max && level <= this.zoom.min){
			return true;
		}
		return false;
	},
	
	setZoom : function(level){
		if(this.isZoomWithinBounds(level)){
			this.zoom.current = level;
			this.refreshBuffer();
		}
		return this;
	},
	
	getZoom : function(){
		return this.zoom.current;
	},
	
	refreshBuffer : function(){
		this.bufferElement = document.createElement('canvas');
		this.bufferElement.width=this.canvasElement.width * this.zoom.current;
		this.bufferElement.height=this.canvasElement.height * this.zoom.current;
		this.buffer = this.bufferElement.getContext('2d');
	},
	
	/**
	 * Get location from tile.
	 * Convert this tile element into [x,y] coordinates where the tile
	 * is to appear on the map.
	 * @see http://experimentgarden.blogspot.com/2009/06/translating-mouse-position-over.html
	 */
	getLocFromTile : function(tile){
		return this.getLoc(tile.x,tile.y,this.map[tile.x][tile.y][0].z);
	},
	
	/**
	 * Get x,y coordinates of a tile location.
	 */
	getLoc : function(tx,ty,tz){
		var px = this.tileDef.tilesPerX * (tx + ty);
		var py = this.tileDef.tilesPerY * (tx - ty);
		
		// Account for a panned map.
		px = px - this.offset[0];
		py = py - this.offset[1];
		
		// Account for a z height
		if(typeof tz != 'undefined' && tz > 0){
			py -= this.tileDef.height/4*tz;
		}
		
		return [px,py];
	},
	
	getLocSub : function(x,y){
		var w = this.tileDef.width;
		var h = this.tileDef.height;
		
		var px = Math.floor(w/this.tileDef.subtiles/2 * (x+y));
		var py = Math.floor(h/24 * (x-y) + h*.75); //This is very obviously not right, but I'm not sure how to fix it.
		return [px,py];
	},
	
	/**
	 * Get tile from location.
	 * Given the x,y location, this function returns the tile that
	 * the cursor has selected.
	 */
	getTileFromLoc : function(px,py){
		
		var pageOffset = IsoRes.getObjectPos(this.canvasElement);
		
		// Compensate for zoom and offset.
		px = px * this.zoom.current + this.offset[0];
		py = py * this.zoom.current + this.offset[1];
		
		// cursor position + map offset + canvas offset (from edge of page)
		px = (px - pageOffset[0]) / this.tileDef.tilesPerX;
		py = (py - pageOffset[1]) / this.tileDef.tilesPerY;
		
		// Tile number
		var tx = (px + py) / 2;
		var ty = (px - py) / 2;
		
		// Subtile
		var stx = Math.round(((tx-2)%1+.5)%1*5);
		var sty = Math.round(((ty+1)%1+.5)%1*5);
		
		tx = Math.round(tx-2);
		ty = Math.round(ty+1);
		
		// Tile x, Tile y, Subtile x, Subtile y
		return [tx,ty,sty,stx];
	},
	
	/**
	 * Get the item (sprite, tile, whatever) from this cursor location.
	 */
	getItemFromLoc : function(px,py){
			
		for(var i=this.sprites.length-1; i>=0; i--){
			var sprite = this.sprites[i];
			if(this.isInMap(sprite.locTile)){
				// if there's a tile in the area, check that pixel
				var locSprite = this.getLoc(sprite.locTile[0],sprite.locTile[1]);
				locSprite[0] += sprite.dx;
				locSprite[1] += sprite.dy;
				if(
					px > locSprite[0] && px < locSprite[0] + sprite.width
					&& py > locSprite[1] && py < locSprite[1] + sprite.height
				){
					return sprite;
				}
					
			}
		}
		
		return this.getTileFromLoc(px,py);
		
	},
	
	getRenderContext : function(target){
		try{
			// This is the canvas element to play with, and its drawing context.
			this.canvasElement = document.getElementById(target);
			this.canvas = this.canvasElement.getContext('2d');
			return true;
		} catch(e) {
			// What, is this Internet Explorer or something?
			return false;
		}
	},
	
	start : function(){
		var parent = this;
		// Loop forever
		(function gameLoop(){
			if(parent.gameSpeed != 0) {
				parent.gameLoop();
			}
			requestAnimFrame(gameLoop,parent.canvasElement);
		})();
	},
	
	gameLoop : function(){
		// If the game is unpaused, do these things.
		if(this.gameSpeed == 0) {
			return;
		}
		this.redrawMap();
		
		// Code stolen from http://io-2011-html5-games.appspot.com/#33
		var wallCurrent = Date.now();
		var wallDelta = (wallCurrent - this.wallLastTimestamp) / 1000;
		this.wallLastTimestamp = wallCurrent;

		var gameDelta = Math.min(wallDelta, this.maxStep);
		this.gameTime += gameDelta;
		
	},
	
	drawTile : function(tile){
		for(var i=0; i < tile.length; i++) {
			var thisTile = tile[i].drawTile(this.gameTime); // Pick and prepare tile.
			this.Renderers[thisTile.renderer].call(this,thisTile);
		}
	},
	
	clearCanvas : function(canvas,element){
		canvas.clearRect(0,0,element.width,element.height);
	},
	
	/**
	 * Redraw the map on the screen.
	 * Also calls two hooks redrawMapBefore and redrawMapAfter
	 */
	redrawMap : function(){
		// This tile is used to indicate a hover state.
		this.clearCanvas(this.canvas,this.canvasElement);
		this.clearCanvas(this.buffer,this.bufferElement);
		this.callHook('redrawMapBeforeTiles');
		
		// Fixme: Could probably calculate which tiles need to be drawn,
		// and loop over that instead.
		
		/* Slightly more verbose, but tilePos is defined here so we
		 * don't create a new object for every tile we loop over.
		 */
		var tilePos = {};
		for(var y=this.mapDef.widthTiles - 1; y>=0; y--){
			for(var x=0; x < this.mapDef.heightTiles; x++){
				this.drawTile(this.map[x][y]);
				tilePos.x = x;
				tilePos.y = y;
				this.callHook('redrawMapDrawTile',tilePos);
			}
		}
		
		this.callHook('redrawMapBeforeDraw');
		
		this.canvas.drawImage(this.bufferElement,0,0,this.canvasElement.width,this.canvasElement.height);
		
		this.callHook('redrawMapAfter');
	},
	
	/**
	 * Is the current tile within the map bounds?
	 * @param {number|array} tx The x coordinate or a [x,y] array.
	 * @param {number} ty y coordinate (optional if parameter 1 is an array.
	 * @return {boolean} Whether the tile is within the map grid.
	 */
	isInMap : function(tx,ty){
		// Convert from an array, if that's what this is.
		if(_.isUndefined(ty) && !_.isUndefined(tx)){
			ty = tx[1];
			tx = tx[0];
		}
		
		if(isNaN(tx) || isNaN(ty)
			|| tx < 0 || tx >= this.mapDef.widthTiles
			|| ty  < 0 || ty >= this.mapDef.heightTiles){
			return false;
		}
		
		return true;
		
	},
	
	/**
	 * Is the current pixel on screen?
	 * Small variance to allow for entire tiles that start partially offscreen.
	 * @param {array} loc an [x,y] array of the current pixel position.
	 * @return {boolean}
	 */
	isOnScreen : function(loc){
		
		// Calculate effective on-screen positions
		var ex = loc[0];
		var ey = loc[1];
		
		
		if(	ex < 0 - this.tileDef.width || ex > this.canvasElement.width*this.zoom.current
			|| ey < 0 - this.tileDef.heightPx || ey > this.canvasElement.height * this.zoom.current){
			return false;
		}
		
		return true;
		
	},
	
	/**
	 * Event handler for the canvas. Could do with a tidy-up.
	 * @param e Event object.
	 * @param action Type of event this is.
	 */
	handlerClick : function(e,action){
		this.cursorPos = IsoRes.getCursorPos(e);
		switch(action) {
			case 'start':
				this.start = Date.now();
				this.startPos = this.cursorPos;
				break;
			case 'stop':
			case 'move':
				this.callHook('mapMove',this.cursorPos);
				if(_.isUndefined(this.startPos)) return false;
				var period = Date.now() - this.start;
				
				var xOffset = this.startPos[0]- this.cursorPos[0];
				var yOffset = this.startPos[1]- this.cursorPos[1];
				this.adjustOffset(xOffset,yOffset);
				
				if(xOffset || yOffset)
					this.inputEvent = 'move';
				
				this.startPos = this.cursorPos;
				if(action=='stop') delete this.startPos;
				break;
			case 'click':
				if(this.inputEvent != 'move') {
					this.callHook('mapClick',this.cursorPos);
				}
					
				this.inputEvent = undefined;
				break;
		}
		e.preventDefault();
	},
	
	setCanvasSize : function(w,h){
		this.canvasElement.width = w;
		this.canvasElement.height = h;
		this.refreshBuffer();
		return this;a
	}
	
}


