

/**
 * Convert this IsoCan instance into a re-instantiable object.
 */
IsoCan.prototype.getSavegame = function(){
	
	var savegame = {map:[]};
	
	for(property in this){
		if(!_.isFunction(this[property])){
			savegame[property] = this[property];
		}
	}
	
	savegame = this.applyFilter('savegameSave',savegame);
	try{
		savegame = JSON.stringify(savegame);
	}catch(e){
		savegame = false;
		console.log(e);
	}
	return savegame;
	
}

IsoCan.prototype.loadSavegame = function(saveText){
	
	try {
		var savegame = JSON.parse(saveText);
	} catch(e) {
		console.log('Can\'t parse savegame file.');
		return false;
	}
	
	savegame = this.applyFilter('savegameLoad',savegame);
	
	// FIXME, need to merge the rest of the game variables into this.
	
	return this;
}



/**
 * Savegame Map Sanitiser Filter
 * Sanitises map tiles and calls the proper "save" function on these 
 * items.
 * @param {object} Savegame object, duplicate of "this".
 * @returns {object} Savegame object.
 */
IsoCan.prototype.Filters.savegameSave.push(function(savegame){
	savegame.map = []
	for(var x=0; x<this.mapDef.widthTiles; x++){
		savegame.map[x] = [];
		for(var y=0; y<this.mapDef.heightTiles; y++){
			savegame.map[x][y] = [];
			for(var z=0; z<this.map[x][y].length; z++){
				//clone the tile and use the internal save function.
				var thisTile = _.clone(this.map[x][y][z]);
				thisTile = thisTile.save();
				
				//manually remove any useless/tricky properties.
				_.each([
					'parent',
					'image',
					'context',
					'z',
					'y',
					'x'
				],function(prop){
					delete thisTile[prop];
				});
				savegame.map[x][y].push(thisTile);
			}
		}
	}
	savegame.sprites = [];
	
	for(var x=0; x<this.sprites.length; x++){
		var thisSprite = this.sprites[x].save();
		savegame.sprites[x] = thisSprite;
	}
	return savegame;
});

/**
 * Savegame Filter out unnecessary elements
 * @param {object} Savegame object, duplicate of "this".
 * @returns {object} Savegame object.
 */
IsoCan.prototype.Filters.savegameSave.push(function(savegame){
	_.each([
		'canvas',
		'canvasElement'
	],function(prop){
		delete savegame[prop];
	});
	return savegame;
});

/**
 * Savegame Map Tile Import Filter
 * Reconstitutes map tiles based on the saved configurations.
 * @param {object} Savegame object, duplicate of "this".
 * @returns {object} Savegame object.
 */
IsoCan.prototype.Filters.savegameLoad.push(function(savegame){
	for(var x=0; x<this.mapDef.widthTiles; x++){
		for(var y=0; y<this.mapDef.heightTiles; y++){
			this.map[x][y] = [];
			for(var z=0; z<savegame.map[x][y].length; z++){
				var flatTile = savegame.map[x][y][z];
				// If we ain't got this tile, cancel the load.
				if(!_.isFunction(IsoRes.Tiles[flatTile.tileName])){
					console.log('Tile ',flatTile.tileName,' not found');
					return false;
				}
				this.setTile(x,y,this.createTile(flatTile.tileName,flatTile));
			}
		}
	}
	delete savegame.map;
	
	
	this.sprites = [];
	if(!_.isUndefined(savegame.sprites)){
		for(var x=0; x<savegame.sprites.length; x++){
			var thisSprite = savegame.sprites[x];
			thisSprite = this.createSprite(thisSprite.spriteName,thisSprite);
			thisSprite.loadSprite();
			this.sprites[x] = thisSprite;
		}
	}
	
	return savegame;
});
