IsoRes.Tiles.Decoration = function(options){
	_.extend(this,options);
	this.style = this.style ? this.style : 'forest';
	
	this.tileName = 'Decoration';
	
	this.image = document.createElement('canvas');
	this.image.width = this.parent.tileDef.width;
	this.image.height = this.parent.tileDef.height;
	this.context = this.image.getContext('2d');
	
	if(!_.isUndefined(this.decorations)){
		this.load();
		return this;
	}
	
	// 6*6 grid on each tile
	this.decorations = [
		[0,0,0,0,0,0],
		[0,0,0,0,0,0],
		[0,0,0,0,0,0],
		[0,0,0,0,0,0],
		[0,0,0,0,0,0],
		[0,0,0,0,0,0]
	];
	
	if(this.style == 'forest')
		this.createForest();
	
}

_.extend(
	IsoRes.Tiles.Decoration.prototype,
	IsoRes.Tiles.Basic.prototype,
	{
		tilePrefix : 'Decoration',
		style : 'forest',
		forestDensity: .4,
		stackable : false,
		styles : {
			'forest' : ['g3649','g4369','g8086','g6786','g8323','g5822','g5857','g5893','g6189','g6271','g6690'],
			'forest-pixel' : ['tree-1','tree-2','tree-3','tree-4','tree-6'],
			'garden' : ['tree-potted-1','tree-potted-2']
		},
		prepareTile : function(){
			this.context.clearRect(0,0,this.image.width,this.image.height);
			for(var x=0; x<this.decorations.length; x++){
				for(var y=this.decorations[x].length-1; y>=0; y--){
					var decoration = this.decorations[x][y];
					if(decoration && !_.isUndefined(decoration.image)){
						loc = this.parent.getLocSub(x,y);
						this.context.drawImage(decoration.image,loc[0],loc[1]-decoration.image.height);
					}
				}
			}
			this.changed = false;
		},
		placeDecoration : function(x,y,decoration){
			var newDecoration = new IsoRes.Decoration({obstacle:true})
				.setImage(this.parent.getImageUri(this.tileName,decoration));
			newDecoration.parent = this;
			this.decorations[x][y] = newDecoration;
			this.changed = true;
		},
		createForest : function(){
			var trees = this.styles.forest;
			for(var i=0; i<6; i++){
				for(j=0;j<6;j++){
					var random = Math.random();
					if(random <= this.forestDensity) {
						var treeType = Math.floor(Math.random()*trees.length);
						this.placeDecoration(i,j,trees[treeType]);
					}
				}
			}
		},
		save : function(){
			var savedTile = _.extend({},this);
			for(var i=0;i<savedTile.decorations.length;i++){
				for(var j=0;j<savedTile.decorations[i].length;j++){
					_.each([
						'parent',
						'image'
					],function(prop){
						delete savedTile.decorations[i][j][prop];
					});
				}
			}
			return savedTile;
		},
		load : function(){
			for(var i=0;i<this.decorations.length;i++){
				for(var j=0;j<this.decorations[i].length;j++){
					this.decorations[i][j] = new IsoRes.Decoration(_.extend(this.decorations[i][j],{parent:this}));
				};
			};
		}
	}
);

IsoRes.Decoration = function(options){
	_.extend(this,options);
	if(this.src){
		this.setImage(this.src);
	}
}
IsoRes.Decoration.prototype = {
	setImage : function(uri){
		this.image = new Image();
		this.src = uri;
		this.image.src = uri;
		var parent = this;
		this.image.onload = function(){
			// If the parent is set, set it to changed so it'll refresh.
			if(!_.isUndefined(parent.parent))
				parent.parent.changed=true;
			
			delete parent.image.onload;
		}
		return this;
	},
	isObstacle : function(){
		if(this.obstacle) return true;
		return false;
	}
}
