IsoRes.Tiles.Road = function(options){
	_.extend(this,options);
	this.style = this.style ? this.style : 'sidewalk';
	this.connections = this.connections ? this.connections : [false,false,false,false];
	
	this.tileName = 'Road';
	this.setCorners(false,false,false,false);
	this.texture = false;
}

IsoRes.Tiles.Road.prototype = {
	
	tilePrefix : 'road',
	stackable : ['Sidewalk'],
	styles : ['sidewalk','service'],
	connectionNames : {
		n : 3,
		e : 0,
		s : 1,
		w : 2
	},
	connectionStyle : 'all',
	connections : false,
	
	determineConnections : function(){
		var that = this;
		
		/**
		 *  Determine if a tile is in a certain position relative to
		 * this tile, and set both tiles' connections if so.
		 * @param thisConnection Direction of the other tile relative to this. Eg. 'n' for north.
		 * @param neighbourConnection Opposite of thisConnection.
		 * @param x The x of the tile to check
		 * @param y The y of the tile to check.
		 */
		var hasConnection = function(thisConnection,neighbourConnection,x,y){
			if(
				_.isUndefined(that.parent.map[x])
				|| _.isUndefined(that.parent.map[x][y])
			) return;
			
			var neighbour = IsoRes.getTileOfType(that.parent.map[x][y],that.tileName);
			if(neighbour && (that.connectionStyle == 'all' || that.connectionStyle.indexOf(neighbour.style) != -1)) {
				that.setConnection(thisConnection,true);
				neighbour.setConnection(neighbourConnection,true);
				neighbour.loadTexture();
				console.log('found connection ',neighbour);
			} else {
				that.setConnection(thisConnection,false);
				console.log('No connections ');
			}
		}
		
		// Set any connections for these tiles accordingly.
		hasConnection('e', 'w', this.x-1, this.y);
		hasConnection('n', 's', this.x, this.y-1);
		hasConnection('w', 'e', this.x+1, this.y);
		hasConnection('s', 'n', this.x, this.y+1);
		
		this.loadTexture();
		
		return this;
	},
	
	setConnection : function(direction,value){
		
		if(this.connections[this.connectionNames[direction]] != value){
			this.connections[this.connectionNames[direction]] = value;
			this.changed = true;
		}
		return this;
		
	},
	
	/**
	 * Join the NESW connections into a string to load the appropriate
	 * tiles from.
	 */
	getConnectionsString : function(){
		var connections = [];
		
		// Work out if our connections are set and map them to a string.
		var i=0;
		for(var j in this.connectionNames){
			if(this.connections[i++]) connections.push(j);
		}
		var connections = connections.join('-');
				
		return connections;
	},
	
	getStyle : function(){
		
		var connections = this.getConnectionsString();
		
		// If there hasn't been any meaningfulconfiguration set, just
		// display this.
		if(!connections){
			return 'basic';
		}
		
		return this.tilePrefix+'-'+this.style+'-'+connections;
	}
	
}

IsoRes.Tiles.Road.prototype = _.extend(
	{},
	IsoRes.Tiles.Tex.prototype,
	IsoRes.Tiles.Road.prototype
);
