/**
 * Textured tile
 */
IsoRes.Tiles.Tex = function(options){
	_.extend(this,options);
	this.style = this.style ? this.style : 'grass';
	
	this.texture = false;
	
	this.image = document.createElement('canvas');
	this.image.width = 10;
	this.image.height = 10;
	
	this.texture = false;
	
	this.tileName = 'Tex';
	this.setStyle(this.style);
	
	this.setCorners(false,false,false,false);
	this.z = 0;
}

IsoRes.Tiles.Tex.prototype = {
	tileClass : 'tex',
	setStyle : function(style){
		this.style = style;
		this.loadTexture();
	},
	
	loadTexture : function(){
		var that = this;
		var texture = new Image();
		texture.onload = function(){
			that.texture = texture;
			that.changed = true;
			that.prepareTile();
		}
		texture.src = this.parent.getImageUri(this.tileName,this.getStyle());
	},
	/**
	 * Set the z (height) axis. This is an array(4) with boolean values.
	 * True indicates the respective corner is raised.
	 */
	setCorners: function(a,b,c,d){
		
		if(typeof this.corners == 'undefined'){
			this.corners = [];
		} else {
			if(this.corners[0] == a && this.corners[1] == b && this.corners[2] == c && this.corners[3] == d){
				return;
			}
		}
		
		if(a&&b&&c&&d){
			a=false;
			b=false;
			c=false;
			d=false;
			this.z++;
		}
		
		this.corners[0] = a;
		this.corners[1] = b;
		this.corners[2] = c;
		this.corners[3] = d;
		this.changed = true;
		
		//~ if(this.x || this.y){
			//~ var siblings = this.parent.map[this.x][this.y];
			//~ for(var i=0; i<siblings.length; i++){
				//~ if(siblings[i].tileClass == this.tileClass){
					//~ siblings[i].setCorners(a,b,c,d);
				//~ }
			//~ }
		//~ }
	},
	
	setCornersByObject : function(corners){
		if(corners.length != 4){
			return;
		}
		this.setCorners(
			corners[0],
			corners[1],
			corners[2],
			corners[3]
		);
	},
	
	/**
	 * Mask the z axis. Specify the values to toggle.
	 */
	maskCorners : function(mask){
		var oldCorners = _.extend([],this.corners);
		for(var i=0; i<4; i++){
			if(mask[i]){
				oldCorners[i] = !oldCorners[i];
			}
		}
		this.setCornersByObject(oldCorners);
	},
	
	prepareTile : function(){
		if(typeof this.corners == 'undefined' || this.texture == false){
			return;
		}
		
		// Use the corners from the base tile.
		if(this.z != 0 && this.x && this.y){
			this.corners = this.parent.map[this.x][this.y][0].corners;
		}
		
		this.image = this.renderTile(
			this.corners[0],
			this.corners[1],
			this.corners[2],
			this.corners[3],
			this.texture
		);
		
		return this;
	},
	
	/**
	 * Manipulates a base texture and draws a tile in the desired
	 * configuration.
	 * @param {boolean} a Is the top-left corner raised?
	 * @param {boolean} b Is the top-right corner raised?
	 * @param {boolean} c Is the bottom-right corner raised?
	 * @param {boolean} d Is the bottom-left corner raised?
	 * @param {tile} The texture to use. Returned tile will be double the size.
	 */
	renderTile : function(a,b,c,d,tile){
		// Make up a new canvas to play on.
		var can = document.createElement('canvas');
		can.width = tile.width*2;
		can.height = tile.height*2;
		var ctx = can.getContext('2d');
		ctx.save();
		
		// Also create a buffer so we can manipulate textures before drawing
		var canBuff = document.createElement('canvas');
		canBuff.width = can.width;
		canBuff.height = can.width;
		var ctxBuff = canBuff.getContext('2d');
		
		// Just to make things more readable.
		var combo = String(a)+b+c+d;
		
		// Here'es the transform(s) for each elevation combo.
		if(combo == 'truetruefalsefalse'){
			// 1100
			ctx.transform(1,-.5,1,1,0,tile.height);
			ctx.drawImage(tile,0,0);
		}else if(combo == 'falsetruetruefalse'){
			// 0110
			ctx.transform(1,-1,1,.5,0,tile.height*1.5);
			ctx.drawImage(tile,0,0);
		}else if(combo == 'falsefalsetruetrue'){
			// 0011
			ctx.transform(1,-.5,1,0,0,tile.height*1.5);
			ctx.drawImage(tile,0,0);
		}else if(combo == 'truefalsefalsetrue'){
			// 1001
			ctx.transform(1,0,1,.5,0,tile.height);
			ctx.drawImage(tile,0,0);
		}else if(combo == 'falsetruefalsefalse'){
			// 0100
			ctx.transform(1,-.5,1,.5,0,tile.height*1.5);
			ctx.drawImage(tile,0,0);
			ctx.restore();
			
			ctxBuff.transform(1,-1,1,1,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			
			ctx.drawImage(canBuff,0,0,tile.width*2,tile.height*1.5,0,0,tile.width*2,tile.height*1.5);
		} else if(combo=='falsefalsetruefalse'){
			// 0010
			ctx.transform(1,-1,1,0,0,tile.height*2);
			ctx.drawImage(tile,0,0);
			ctx.restore();
			ctx.clearRect(0,tile.height,tile.width,tile.height);
			
			ctxBuff.transform(1,-.5,1,.5,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			ctx.drawImage(canBuff,0,tile.height,tile.width,tile.height,0,tile.height,tile.width,tile.height);
		} else if(combo=='truefalsefalsefalse'){
			// 1000
			ctx.transform(1,0,1,1,0,tile.height);
			ctx.drawImage(tile,0,0);
			ctx.restore();
			ctx.clearRect(tile.width,tile.height,tile.width,tile.height);
			
			ctxBuff.transform(1,-.5,1,.5,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			ctx.drawImage(canBuff,tile.width,tile.height,tile.width,tile.height,tile.width,tile.height,tile.width,tile.height);
		} else if(combo=='truetruetruefalse'){
			// 1110
			ctx.transform(1,-.5,1,1,0,tile.height);
			ctx.drawImage(tile,0,0);
			ctx.restore();
			ctx.clearRect(tile.width,0,tile.width,tile.height*2);
			
			ctxBuff.transform(1,-1,1,.5,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			ctx.drawImage(canBuff,tile.width,0,tile.width,tile.height*2,tile.width,0,tile.width,tile.height*2);
		} else if(combo=='truetruefalsetrue'){
			// 1101
			ctxBuff.save();
			ctxBuff.transform(1,-.5,1,1,0,tile.height);
			ctxBuff.drawImage(tile,0,0);
			
			/* Mask the tile so as to only draw the required portion */
			ctxBuff.globalCompositeOperation = 'destination-in';
			ctxBuff.beginPath();
			ctxBuff.moveTo(0,0);
			ctxBuff.lineTo(tile.width,0);
			ctxBuff.lineTo(tile.width,tile.height);
			ctxBuff.fill();
			
			ctxBuff.restore();
			ctx.drawImage(canBuff,0,0);
			
			ctxBuff.transform(1,0,1,.5,0,tile.height);
			ctxBuff.drawImage(tile,0,0);
			ctxBuff.globalCompositeOperation = 'destination-in';
			
			ctxBuff.beginPath();
			ctxBuff.moveTo(0,0);
			ctxBuff.lineTo(tile.width,tile.height);
			ctxBuff.lineTo(0,tile.height);
			ctxBuff.fill();
			
			ctx.drawImage(canBuff,0,0);
			
		} else if(combo == 'truefalsetruetrue'){
			// 1011
			ctxBuff.save();
			ctxBuff.transform(1,0,-1,.5,tile.width,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			
			/* Mask the tile so as to only draw the required portion */
			ctxBuff.globalCompositeOperation = 'destination-in';
			ctxBuff.beginPath();
			ctxBuff.moveTo(tile.width,0);
			ctxBuff.lineTo(0,0);
			ctxBuff.lineTo(tile.width,tile.height);
			ctxBuff.fill();
			
			ctxBuff.restore();
			ctx.drawImage(canBuff,0,0);
			
			
			ctxBuff.transform(1,0,1,.5,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			
			/* Mask the tile so as to only draw the required portion */
			ctxBuff.globalCompositeOperation = 'destination-in';
			ctxBuff.beginPath();
			ctxBuff.moveTo(0,tile.height);
			ctxBuff.lineTo(0,0);
			ctxBuff.lineTo(tile.width,0);
			ctxBuff.fill();
			
			ctx.drawImage(canBuff,0,0);
		} else if(combo == 'falsetruetruetrue'){
			// 0111
			ctxBuff.save();
			ctxBuff.transform(1,-1,1,.5,0,tile.height*1.5);
			ctxBuff.drawImage(tile,0,0);
			
			/* Mask the tile so as to only draw the required portion */
			ctxBuff.globalCompositeOperation = 'destination-in';
			ctxBuff.beginPath();
			ctxBuff.moveTo(tile.width,0);
			ctxBuff.lineTo(0,0);
			ctxBuff.lineTo(tile.width,tile.height);
			ctxBuff.fill();
			
			ctxBuff.restore();
			ctx.drawImage(canBuff,0,0);
			
			
			ctxBuff.save();
			ctxBuff.transform(1,0,-1,.5,tile.width,tile.height);
			ctxBuff.drawImage(tile,0,0);
			
			/* Mask the tile so as to only draw the required portion */
			ctxBuff.globalCompositeOperation = 'destination-in';
			ctxBuff.beginPath();
			ctxBuff.moveTo(tile.width,0);
			ctxBuff.lineTo(tile.width,tile.height);
			ctxBuff.lineTo(0,tile.height);
			ctxBuff.fill();
			
			ctx.drawImage(canBuff,0,0);
		} else {
			// combo == 'falsefalsefalsefalse'
			// 0000
			// 0001
			ctx.transform(1,-.5,1,.5,0,tile.height*1.5);
			ctx.drawImage(tile,0,0);
		}
		
		return can;
		
	}

}


IsoRes.Tiles.Tex.prototype = _.extend({},IsoRes.Tiles.Basic.prototype,IsoRes.Tiles.Tex.prototype);
